@@ -1,12 +1,16 @@
+from sqlalchemy import and_
 from sqlalchemy import ForeignKey
 from sqlalchemy import inspect
 from sqlalchemy import Integer
+from sqlalchemy import join
 from sqlalchemy import String
 from sqlalchemy import testing
+from sqlalchemy.orm import aliased
 from sqlalchemy.orm import Bundle
 from sqlalchemy.orm import configure_mappers
 from sqlalchemy.orm import defaultload
 from sqlalchemy.orm import defer
+from sqlalchemy.orm import join as orm_join
 from sqlalchemy.orm import joinedload
 from sqlalchemy.orm import Load
 from sqlalchemy.orm import mapper
@@ -21,6 +25,8 @@ from sqlalchemy.testing.schema import Table
 
 
 class MergeTest(fixtures.MappedTest):
+    __requires__ = ("python_profiling_backend",)
+
     @classmethod
     def define_tables(cls, metadata):
         Table(
@@ -93,7 +99,7 @@ class MergeTest(fixtures.MappedTest):
 
         # third call, merge object already present. almost no calls.
 
-        @profiling.function_call_count(variance=0.10)
+        @profiling.function_call_count(variance=0.10, warmup=1)
         def go2():
             return sess2.merge(p2, load=False)
 
@@ -111,7 +117,7 @@ class MergeTest(fixtures.MappedTest):
         # using sqlite3 the C extension took it back up to approx. 1257
         # (py2.6)
 
-        @profiling.function_call_count()
+        @profiling.function_call_count(variance=0.10)
         def go():
             sess2.merge(p1)
 
@@ -137,6 +143,8 @@ class LoadManyToOneFromIdentityTest(fixtures.MappedTest):
 
     """
 
+    __requires__ = ("python_profiling_backend",)
+
     @classmethod
     def define_tables(cls, metadata):
         Table(
@@ -222,6 +230,8 @@ class LoadManyToOneFromIdentityTest(fixtures.MappedTest):
 
 
 class MergeBackrefsTest(fixtures.MappedTest):
+    __requires__ = ("python_profiling_backend",)
+
     @classmethod
     def define_tables(cls, metadata):
         Table(
@@ -314,6 +324,8 @@ class MergeBackrefsTest(fixtures.MappedTest):
 
 
 class DeferOptionsTest(fixtures.MappedTest):
+    __requires__ = ("python_profiling_backend",)
+
     @classmethod
     def define_tables(cls, metadata):
         Table(
@@ -376,6 +388,8 @@ class DeferOptionsTest(fixtures.MappedTest):
 
 
 class AttributeOverheadTest(fixtures.MappedTest):
+    __requires__ = ("python_profiling_backend",)
+
     @classmethod
     def define_tables(cls, metadata):
         Table(
@@ -453,6 +467,8 @@ class AttributeOverheadTest(fixtures.MappedTest):
 
 
 class SessionTest(fixtures.MappedTest):
+    __requires__ = ("python_profiling_backend",)
+
     @classmethod
     def define_tables(cls, metadata):
         Table(
@@ -517,6 +533,8 @@ class SessionTest(fixtures.MappedTest):
 
 
 class QueryTest(fixtures.MappedTest):
+    __requires__ = ("python_profiling_backend",)
+
     @classmethod
     def define_tables(cls, metadata):
         Table(
@@ -580,6 +598,8 @@ class SelectInEagerLoadTest(fixtures.MappedTest):
 
     """
 
+    __requires__ = ("python_profiling_backend",)
+
     @classmethod
     def define_tables(cls, metadata):
 
@@ -658,6 +678,8 @@ class SelectInEagerLoadTest(fixtures.MappedTest):
 
 
 class JoinedEagerLoadTest(fixtures.MappedTest):
+    __requires__ = ("python_profiling_backend",)
+
     @classmethod
     def define_tables(cls, metadata):
         def make_some_columns():
@@ -814,7 +836,98 @@ class JoinedEagerLoadTest(fixtures.MappedTest):
         go()
 
 
+class JoinConditionTest(fixtures.DeclarativeMappedTest):
+    __requires__ = ("python_profiling_backend",)
+
+    @classmethod
+    def setup_classes(cls):
+        class A(cls.DeclarativeBasic):
+            __tablename__ = "a"
+
+            id = Column(Integer, primary_key=True)
+            b_id = Column(ForeignKey("b.id"))
+            b = relationship("B")
+
+        class B(cls.DeclarativeBasic):
+            __tablename__ = "b"
+
+            id = Column(Integer, primary_key=True)
+            d_id = Column(ForeignKey("d.id"))
+
+        class C(cls.DeclarativeBasic):
+            __tablename__ = "c"
+
+            id = Column(Integer, primary_key=True)
+            a_id = Column(ForeignKey("a.id"))
+            d_id = Column(ForeignKey("d.id"))
+
+        class D(cls.DeclarativeBasic):
+            __tablename__ = "d"
+
+            id = Column(Integer, primary_key=True)
+
+        j = join(B, D, B.d_id == D.id).join(C, C.d_id == D.id)
+
+        A.d = relationship(
+            "D",
+            secondary=j,
+            primaryjoin=and_(A.b_id == B.id, A.id == C.a_id),
+            secondaryjoin=D.id == B.d_id,
+            uselist=False,
+            viewonly=True,
+        )
+
+    def test_a_to_b_plain(self):
+        A, B = self.classes("A", "B")
+
+        # should not use aliasing or adaption so should be cheap
+        @profiling.function_call_count(times=50, warmup=1)
+        def go():
+            orm_join(A, B, A.b)
+
+        go()
+
+    def test_a_to_b_aliased(self):
+        A, B = self.classes("A", "B")
+
+        a1 = aliased(A)
+
+        # uses aliasing, therefore adaption which is expensive
+        @profiling.function_call_count(times=50, warmup=1)
+        def go():
+            orm_join(a1, B, a1.b)
+
+        go()
+
+    def test_a_to_d(self):
+        A, D = self.classes("A", "D")
+
+        # the join condition between A and D uses a secondary selectable  with
+        # overlap so incurs aliasing, which is expensive, there is also a check
+        # that determines that this overlap exists which is not currently
+        # cached
+        @profiling.function_call_count(times=50, warmup=1)
+        def go():
+            orm_join(A, D, A.d)
+
+        go()
+
+    def test_a_to_d_aliased(self):
+        A, D = self.classes("A", "D")
+
+        a1 = aliased(A)
+
+        # aliased, uses adaption therefore expensive
+        @profiling.function_call_count(times=50, warmup=1)
+        def go():
+            orm_join(a1, D, a1.d)
+
+        go()
+
+
 class BranchedOptionTest(fixtures.MappedTest):
+    __requires__ = ("python_profiling_backend",)
+
     @classmethod
     def define_tables(cls, metadata):
         def make_some_columns():
@@ -943,7 +1056,7 @@ class BranchedOptionTest(fixtures.MappedTest):
 
         cache_path = inspect(A)._path_registry
 
-        @profiling.function_call_count()
+        @profiling.function_call_count(warmup=1)
         def go():
             for opt in opts:
                 opt._generate_cache_key(cache_path)
@@ -963,7 +1076,7 @@ class BranchedOptionTest(fixtures.MappedTest):
 
         cache_path = inspect(A)._path_registry
 
-        @profiling.function_call_count()
+        @profiling.function_call_count(warmup=1)
         def go():
             for opt in opts:
                 opt._generate_cache_key(cache_path)
@@ -983,7 +1096,7 @@ class BranchedOptionTest(fixtures.MappedTest):
 
         q = Session().query(A)
 
-        @profiling.function_call_count()
+        @profiling.function_call_count(warmup=1)
         def go():
             q.options(*opts)
 
@@ -1002,7 +1115,7 @@ class BranchedOptionTest(fixtures.MappedTest):
 
         q = Session().query(A)
 
-        @profiling.function_call_count()
+        @profiling.function_call_count(warmup=1)
         def go():
             q.options(*opts)
 
@@ -1010,6 +1123,8 @@ class BranchedOptionTest(fixtures.MappedTest):
 
 
 class AnnotatedOverheadTest(fixtures.MappedTest):
+    __requires__ = ("python_profiling_backend",)
+
     @classmethod
     def define_tables(cls, metadata):
         Table(
@@ -1046,7 +1161,7 @@ class AnnotatedOverheadTest(fixtures.MappedTest):
 
         q = s.query(A).select_from(A)
 
-        @profiling.function_call_count()
+        @profiling.function_call_count(warmup=1)
         def go():
             for i in range(100):
                 q.all()
@@ -1060,7 +1175,7 @@ class AnnotatedOverheadTest(fixtures.MappedTest):
 
         q = s.query(a.c.data).select_from(A)
 
-        @profiling.function_call_count()
+        @profiling.function_call_count(warmup=1)
         def go():
             for i in range(100):
                 q.all()
@@ -1072,7 +1187,7 @@ class AnnotatedOverheadTest(fixtures.MappedTest):
         s = Session()
         q = s.query(A.data).select_from(A)
 
-        @profiling.function_call_count()
+        @profiling.function_call_count(warmup=1)
         def go():
             for i in range(100):
                 q.all()
@@ -1084,7 +1199,7 @@ class AnnotatedOverheadTest(fixtures.MappedTest):
         s = Session()
         q = s.query(A, A.data).select_from(A)
 
-        @profiling.function_call_count()
+        @profiling.function_call_count(warmup=1)
         def go():
             for i in range(100):
                 q.all()
@@ -1097,7 +1212,7 @@ class AnnotatedOverheadTest(fixtures.MappedTest):
         s = Session()
         q = s.query(A, a.c.data).select_from(A)
 
-        @profiling.function_call_count()
+        @profiling.function_call_count(warmup=1)
         def go():
             for i in range(100):
                 q.all()
@@ -1110,7 +1225,7 @@ class AnnotatedOverheadTest(fixtures.MappedTest):
         s = Session()
         q = s.query(a.c.data, A).select_from(A)
 
-        @profiling.function_call_count()
+        @profiling.function_call_count(warmup=1)
         def go():
             for i in range(100):
                 q.all()
@@ -1122,7 +1237,7 @@ class AnnotatedOverheadTest(fixtures.MappedTest):
         s = Session()
         q = s.query(A.data, A).select_from(A)
 
-        @profiling.function_call_count()
+        @profiling.function_call_count(warmup=1)
         def go():
             for i in range(100):
                 q.all()
@@ -1135,7 +1250,7 @@ class AnnotatedOverheadTest(fixtures.MappedTest):
         s = Session()
         q = s.query(Bundle("ASdf", a.c.data), A).select_from(A)
 
-        @profiling.function_call_count()
+        @profiling.function_call_count(warmup=1)
         def go():
             for i in range(100):
                 q.all()
@@ -1147,7 +1262,7 @@ class AnnotatedOverheadTest(fixtures.MappedTest):
         s = Session()
         q = s.query(Bundle("ASdf", A.data), A).select_from(A)
 
-        @profiling.function_call_count()
+        @profiling.function_call_count(warmup=1)
         def go():
             for i in range(100):
                 q.all()
