@@ -7,12 +7,14 @@ import sys
 
 from sqlalchemy import exc
 from sqlalchemy import util
+from sqlalchemy.sql import text
 from sqlalchemy.testing import exclusions
 from sqlalchemy.testing.exclusions import against
 from sqlalchemy.testing.exclusions import fails_if
 from sqlalchemy.testing.exclusions import fails_on
 from sqlalchemy.testing.exclusions import fails_on_everything_except
 from sqlalchemy.testing.exclusions import LambdaPredicate
+from sqlalchemy.testing.exclusions import NotPredicate
 from sqlalchemy.testing.exclusions import only_if
 from sqlalchemy.testing.exclusions import only_on
 from sqlalchemy.testing.exclusions import skip_if
@@ -632,6 +634,23 @@ class DefaultRequirements(SuiteRequirements):
     def two_phase_transactions(self):
         """Target database must support two-phase transactions."""
 
+        def pg_prepared_transaction(config):
+            if not against(config, "postgresql"):
+                return False
+
+            with config.db.connect() as conn:
+                try:
+                    num = conn.scalar(
+                        text(
+                            "select cast(setting AS integer) from pg_settings "
+                            "where name = 'max_prepared_transactions'"
+                        )
+                    )
+                except exc.OperationalError:
+                    return False
+                else:
+                    return num > 0
+
         return skip_if(
             [
                 no_support("firebird", "no SA implementation"),
@@ -658,6 +677,12 @@ class DefaultRequirements(SuiteRequirements):
                     "recent MySQL communiity editions have too many issues "
                     "(late 2016), disabling for now",
                 ),
+                NotPredicate(
+                    LambdaPredicate(
+                        pg_prepared_transaction,
+                        "max_prepared_transactions not available or zero",
+                    )
+                ),
             ]
         )
 
@@ -1287,8 +1312,8 @@ class DefaultRequirements(SuiteRequirements):
                 drivername = conn.connection.connection.getinfo(
                     config.db.dialect.dbapi.SQL_DRIVER_NAME
                 )
-                # on linux this is 'libmsodbcsql-13.1.so.9.2'.
-                # don't know what it is on windows
+                # on linux this is something like 'libmsodbcsql-13.1.so.9.2'.
+                # on Windows this is something like 'msodbcsql17.dll'.
                 return "msodbc" in drivername
 
         return only_if(
@@ -1302,6 +1327,19 @@ class DefaultRequirements(SuiteRequirements):
             "Python issue 8743 fixed in Python 2.7.8",
         )
 
+    @property
+    def granular_timezone(self):
+        """the datetime.timezone class, or SQLAlchemy's port, supports
+        seconds and microseconds.
+
+        SQLAlchemy ported the Python 3.7 version for Python 2, so
+        it passes on that.  For Python 3.6 and earlier, it is not supported.
+
+        """
+        return exclusions.skip_if(
+            lambda: sys.version_info >= (3,) and sys.version_info < (3, 7)
+        )
+
     @property
     def selectone(self):
         """target driver must support the literal statement 'select 1'"""
@@ -1403,6 +1441,12 @@ class DefaultRequirements(SuiteRequirements):
             or config.db.dialect._mariadb_normalized_version_info < (10, 3)
         )
 
+    def _mysql_not_mariadb_104(self, config):
+        return against(config, "mysql") and (
+            not config.db.dialect._is_mariadb
+            or config.db.dialect._mariadb_normalized_version_info < (10, 4)
+        )
+
     def _has_mysql_on_windows(self, config):
         return (
             against(config, "mysql")
@@ -1439,3 +1483,7 @@ class DefaultRequirements(SuiteRequirements):
     @property
     def computed_columns(self):
         return skip_if(["postgresql < 12", "sqlite", "mysql < 5.7"])
+
+    @property
+    def python_profiling_backend(self):
+        return only_on(["sqlite"])
